@@ -1,4 +1,6 @@
 """This modules defines all kinds of exceptions raised in Jina."""
+from typing import Set, Union
+
 import grpc.aio
 
 
@@ -34,6 +36,10 @@ class BadClient(Exception, BaseJinaException):
     """A wrongly defined client, can not communicate with jina server correctly."""
 
 
+class BadServer(Exception, BaseJinaException):
+    """Error happens on the server side."""
+
+
 class BadClientCallback(BadClient, BaseJinaException):
     """Error in the callback function on the client side."""
 
@@ -84,13 +90,13 @@ class InternalNetworkError(grpc.aio.AioRpcError, BaseJinaException):
         self,
         og_exception: grpc.aio.AioRpcError,
         request_id: str = '',
-        dest_addr: str = '',
+        dest_addr: Union[str, Set[str]] = {''},
         details: str = '',
     ):
         """
         :param og_exception: the original exception that caused the network error
         :param request_id: id of the request that caused the error
-        :param dest_addr: destination (microservice) address of the problematic network call
+        :param dest_addr: destination (microservice) address(es) of the problematic network call(s)
         :param details: details of the error
         """
         self.og_exception = og_exception
