@@ -1,23 +1,32 @@
+import contextlib
 import inspect
 import multiprocessing
 import os
 import threading
 import warnings
 from types import SimpleNamespace
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, Union
+from typing import TYPE_CHECKING, Any, Dict, Optional, Type, Union
 
 from jina import __args_executor_init__, __default_endpoint__
 from jina.enums import BetterEnum
 from jina.helper import ArgNamespace, T, iscoroutinefunction, typename
 from jina.importer import ImportExtensions
 from jina.jaml import JAML, JAMLCompatible, env_var_regex, internal_var_regex
-from jina.serve.executors.decorators import requests, store_init_kwargs, wrap_func
+from jina.logging.logger import JinaLogger
+from jina.serve.executors.decorators import (
+    avoid_concurrent_lock_cls,
+    requests,
+    store_init_kwargs,
+    wrap_func,
+)
 
 if TYPE_CHECKING:
-    from jina import DocumentArray
+    from docarray import DocumentArray
+    from prometheus_client import Summary
 
+__dry_run_endpoint__ = '_jina_dry_run_'
 
-__all__ = ['BaseExecutor', 'ReducerExecutor']
+__all__ = ['BaseExecutor', __dry_run_endpoint__]
 
 
 class ExecutorType(type(JAMLCompatible), type):
@@ -56,6 +65,7 @@ class ExecutorType(type(JAMLCompatible), type):
                     f'please add `**kwargs` to your __init__ function'
                 )
             wrap_func(cls, ['__init__'], store_init_kwargs)
+            wrap_func(cls, ['__init__'], avoid_concurrent_lock_cls(cls))
 
             reg_cls_set.add(cls_id)
             setattr(cls, '_registered_class', reg_cls_set)
@@ -64,7 +74,24 @@ class ExecutorType(type(JAMLCompatible), type):
 
 class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
     """
-    The base class of the executor, can be used to build encoder, indexer, etc.
+    The base class of all Executors, can be used to build encoder, indexer, etc.
+
+    :class:`jina.Executor` as an alias for this class.
+
+    EXAMPLE USAGE
+
+    .. code-block:: python
+
+        from jina import Executor, requests, Flow
+
+
+        class MyExecutor(Executor):
+            @requests
+            def foo(self, docs, **kwargs):
+                print(docs)  # process docs here
+
+
+        f = Flow().add(uses=Executor)  # you can add your Executor to a Flow
 
     Any executor inherited from :class:`BaseExecutor` always has the **meta** defined in :mod:`jina.executors.metas.defaults`.
 
@@ -73,7 +100,7 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
     .. highlight:: python
     .. code-block:: python
 
-        class MyAwesomeExecutor:
+        class MyAwesomeExecutor(Executor):
             def __init__(awesomeness=5):
                 pass
 
@@ -106,6 +133,17 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
         self._add_requests(requests)
         self._add_runtime_args(runtime_args)
         self._init_monitoring()
+        self.logger = JinaLogger(self.__class__.__name__)
+        if __dry_run_endpoint__ not in self.requests:
+            self.requests[__dry_run_endpoint__] = self._dry_run_func
+        else:
+            self.logger.warning(
+                f' Endpoint {__dry_run_endpoint__} is defined by the Executor. Be aware that this endpoint is usually reserved to enable health checks from the Client through the gateway.'
+                f' So it is recommended not to expose this endpoint. '
+            )
+
+    def _dry_run_func(self, *args, **kwargs):
+        pass
 
     def _add_runtime_args(self, _runtime_args: Optional[Dict]):
         if _runtime_args:
@@ -129,10 +167,13 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
                 'Time spent when calling the executor request method',
                 registry=self.runtime_args.metrics_registry,
                 namespace='jina',
-                labelnames=('method', 'executor'),
+                labelnames=('executor', 'executor_endpoint', 'runtime_name'),
             )
+            self._metrics_buffer = {'process_request_seconds': self._summary_method}
+
         else:
             self._summary_method = None
+            self._metrics_buffer = None
 
     def _add_requests(self, _requests: Optional[Dict]):
         if not hasattr(self, 'requests'):
@@ -250,10 +291,24 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
 
     async def __acall_endpoint__(self, req_endpoint, **kwargs):
         func = self.requests[req_endpoint]
-        if iscoroutinefunction(func):
-            return await func(self, **kwargs)
-        else:
-            return func(self, **kwargs)
+
+        runtime_name = (
+            self.runtime_args.name if hasattr(self.runtime_args, 'name') else None
+        )
+
+        _summary = (
+            self._summary_method.labels(
+                self.__class__.__name__, req_endpoint, runtime_name
+            ).time()
+            if self._summary_method
+            else contextlib.nullcontext()
+        )
+
+        with _summary:
+            if iscoroutinefunction(func):
+                return await func(self, **kwargs)
+            else:
+                return func(self, **kwargs)
 
     @property
     def workspace(self) -> Optional[str]:
@@ -383,7 +438,7 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
         SHARED = 1  # not served by a gateway, served by head/worker
 
     @staticmethod
-    def to_k8s_yaml(
+    def to_kubernetes_yaml(
         uses: str,
         output_base_path: str,
         k8s_namespace: Optional[str] = None,
@@ -412,19 +467,20 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
         """
         from jina import Flow
 
-        f = Flow(**kwargs).add(
+        Flow(**kwargs).add(
             uses=uses,
             uses_with=uses_with,
             uses_metas=uses_metas,
             uses_requests=uses_requests,
-        )
-        f.to_k8s_yaml(
+        ).to_kubernetes_yaml(
             output_base_path=output_base_path,
             k8s_namespace=k8s_namespace,
             include_gateway=executor_type
             == BaseExecutor.StandaloneExecutorType.EXTERNAL,
         )
 
+    to_k8s_yaml = to_kubernetes_yaml
+
     @staticmethod
     def to_docker_compose_yaml(
         uses: str,
@@ -464,24 +520,28 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
             == BaseExecutor.StandaloneExecutorType.EXTERNAL,
         )
 
-
-class ReducerExecutor(BaseExecutor):
-    """
-    ReducerExecutor is an Executor that performs a reduce operation on a matrix of DocumentArrays coming from shards.
-    ReducerExecutor relies on DocumentArray.reduce_all to merge all DocumentArray into one DocumentArray which will be
-    sent to the next deployment.
-
-    This Executor only adds a reduce endpoint to the BaseExecutor.
-    """
-
-    @requests
-    def reduce(self, docs_matrix: List['DocumentArray'] = [], **kwargs):
-        """Reduce docs_matrix into one `DocumentArray` using `DocumentArray.reduce_all`
-        :param docs_matrix: a List of DocumentArrays to be reduced
-        :param kwargs: extra keyword arguments
-        :return: the reduced DocumentArray
+    def monitor(
+        self, name: Optional[str] = None, documentation: Optional[str] = None
+    ) -> Optional['Summary']:
         """
-        if docs_matrix:
-            da = docs_matrix[0]
-            da.reduce_all(docs_matrix[1:])
-            return da
+        Get a given prometheus metric, if it does not exist yet, it will create it and store it in a buffer.
+        :param name: the name of the metrics
+        :param documentation:  the description of the metrics
+
+        :return: the given prometheus metrics or None if monitoring is not enable.
+        """
+
+        if self._metrics_buffer:
+            if name not in self._metrics_buffer:
+                from prometheus_client import Summary
+
+                self._metrics_buffer[name] = Summary(
+                    name,
+                    documentation,
+                    registry=self.runtime_args.metrics_registry,
+                    namespace='jina',
+                    labelnames=('runtime_name',),
+                ).labels(self.runtime_args.name)
+            return self._metrics_buffer[name].time()
+        else:
+            return contextlib.nullcontext()
